@@ -23,6 +23,7 @@ from collections import namedtuple
 from math import ceil, trunc
 
 from testutil import passed, failBecause, testing_metrics
+from term_color import Color, colored
 
 from my_typing import *
 
@@ -45,9 +46,18 @@ def is_worktree_dirty() -> bool:
     return subprocess.check_output(['git', 'status', '--porcelain']) != b''
 
 #
-# Some data access functions. A the moment this uses git notes.
+# Some data access functions. At the moment this uses git notes.
 #
 
+NoteNamespace = NewType("NoteNamespace", str)
+
+# The git notes namespace for local results.
+LocalNamespace = NoteNamespace("perf")
+
+# The git notes namespace for ci results.
+CiNamespace = NoteNamespace("ci/" + LocalNamespace)
+
+
 # The metrics (a.k.a stats) are named tuples, PerfStat, in this form:
 #
 # ( test_env : 'val',      # Test environment.
@@ -66,15 +76,30 @@ PerfStat = NamedTuple('PerfStat', [('test_env', TestEnv),
 
 # A baseline recovered form stored metrics.
 Baseline = NamedTuple('Baseline', [('perfStat', PerfStat),
-                                   ('commit', GitHash),
-                                   ('commitDepth', int)])
+                                   ('commit', GitHash)])
 
 class MetricChange(Enum):
+    # The metric appears to have no baseline and is presumably a new test.
     NewMetric = 'NewMetric'
+
+    # The metric has not changed.
     NoChange = 'NoChange'
+
+    # The metric increased.
     Increase = 'Increase'
+
+    # The metric decreased.
     Decrease = 'Decrease'
 
+    def __str__(self):
+        strings = {
+            MetricChange.NewMetric: colored(Color.BLUE,  "new"),
+            MetricChange.NoChange:  colored(Color.WHITE, "unchanged"),
+            MetricChange.Increase:  colored(Color.RED,   "increased"),
+            MetricChange.Decrease:  colored(Color.GREEN, "decreased")
+        }
+        return strings[self]
+
 AllowedPerfChange = NamedTuple('AllowedPerfChange',
                                [('direction', MetricChange),
                                 ('metrics', List[str]),
@@ -88,22 +113,29 @@ MetricOracles = NamedTuple("MetricOracles", [("baseline", MetricBaselineOracle),
 
 def parse_perf_stat(stat_str: str) -> PerfStat:
     field_vals = stat_str.strip('\t').split('\t')
-    return PerfStat(*field_vals) # type: ignore
+    stat = PerfStat(*field_vals) # type: ignore
+    if stat.test_env.startswith('"') and stat.test_env.endswith('"'):
+        # Due to a bug, in historical data sometimes the test_env
+        # contains additional quotation marks (#18656).
+        # Remove them, so that we can refer to past data in a uniform fashion.
+        stat = stat._replace(test_env=TestEnv(stat.test_env[1:-1]))
+    return stat
 
 # Get all recorded (in a git note) metrics for a given commit.
 # Returns an empty array if the note is not found.
-def get_perf_stats(commit: GitRef=GitRef('HEAD'),
-                   namespace: str='perf'
+def get_perf_stats(commit: Union[GitRef, GitHash]=GitRef('HEAD'),
+                   namespace: NoteNamespace = LocalNamespace
                    ) -> List[PerfStat]:
     try:
         log = subprocess.check_output(['git', 'notes', '--ref=' + namespace, 'show', commit], stderr=subprocess.STDOUT).decode('utf-8')
     except subprocess.CalledProcessError:
         return []
 
-    log = log.strip('\n').split('\n')
-    log = list(filter(None, log))
-    log = [parse_perf_stat(stat_str) for stat_str in log]
-    return log
+    return \
+        [ parse_perf_stat(stat_str)
+          for stat_str in log.strip('\n').split('\n')
+          if stat_str != ''
+        ]
 
 # Check if a str is in a 40 character git commit hash.
 _commit_hash_re = re.compile('[0-9a-f]' * 40)
@@ -134,7 +166,7 @@ def commit_hash(commit: Union[GitHash, GitRef]) -> GitHash:
 #               }
 #   }
 _get_allowed_perf_changes_cache = {} # type: Dict[GitHash, Dict[TestName, List[AllowedPerfChange]]]
-def get_allowed_perf_changes(commit: GitRef=GitRef('HEAD')
+def get_allowed_perf_changes(commit: Union[GitRef, GitHash]=GitRef('HEAD')
                              ) -> Dict[TestName, List[AllowedPerfChange]]:
     global _get_allowed_perf_changes_cache
     chash = commit_hash(commit)
@@ -188,7 +220,7 @@ def parse_allowed_perf_changes(commitMsg: str
 # Calculates a suggested string to append to the git commit in order to accept the
 # given changes.
 # changes: [(MetricChange, PerfStat)]
-def allow_changes_string(changes: List[Tuple[MetricChange, PerfStat, Any]]
+def allow_changes_string(changes: List[Tuple[MetricChange, PerfStat]]
                          ) -> str:
     Dec = MetricChange.Decrease
     Inc = MetricChange.Increase
@@ -198,7 +230,7 @@ def allow_changes_string(changes: List[Tuple[MetricChange, PerfStat, Any]]
 
     # Map tests to a map from change direction to metrics.
     test_to_dir_to_metrics = {} # type: Dict[TestName, Dict[MetricChange, List[MetricName]]]
-    for (change, perf_stat, _baseline) in changes:
+    for (change, perf_stat) in changes:
         change_dir_to_metrics = test_to_dir_to_metrics.setdefault(perf_stat.test, { Inc: [], Dec: [] })
         change_dir_to_metrics[change].append(perf_stat.metric)
 
@@ -224,11 +256,11 @@ def allow_changes_string(changes: List[Tuple[MetricChange, PerfStat, Any]]
 
     # Decreasing group.
     if groupDec:
-        msgs.append('Metric Decrease:' + nltab + nltab.join(groupDec))
+        msgs.append('Metric Decrease:' + nltab + nltab.join(sorted(groupDec)))
 
     # Increasing group.
     if groupInc:
-        msgs.append('Metric Increase:' + nltab + nltab.join(groupInc))
+        msgs.append('Metric Increase:' + nltab + nltab.join(sorted(groupInc)))
 
     # Mixed group.
     if groupMix:
@@ -245,7 +277,7 @@ def allow_changes_string(changes: List[Tuple[MetricChange, PerfStat, Any]]
         for change_dir in [Dec, Inc]:
             metric_to_tests = dir_to_metric_to_tests[change_dir]
             for metric in sorted(metric_to_tests.keys()):
-                tests = metric_to_tests[metric]
+                tests = sorted(metric_to_tests[metric])
                 msgs.append('Metric ' + change_dir.value + ' \'' + metric + '\':' + nltab + nltab.join(tests))
 
     return '\n\n'.join(msgs)
@@ -264,7 +296,7 @@ def format_perf_stat(stats: Union[PerfStat, List[PerfStat]], delimitor: str = "\
 # Returns True if the note was successfully appended.
 def append_perf_stat(stats: List[PerfStat],
                      commit: GitRef = GitRef('HEAD'),
-                     namespace: str ='perf',
+                     namespace: NoteNamespace = LocalNamespace,
                      max_tries: int=5
                      ) -> bool:
     # Append to git note
@@ -295,12 +327,6 @@ def append_perf_stat(stats: List[PerfStat],
 # Max number of ancestor commits to search when compiling a baseline performance metric.
 BaselineSearchDepth = 75
 
-# The git notes name space for local results.
-LocalNamespace = "perf"
-
-# The git notes name space for ci results.
-CiNamespace = "ci/" + LocalNamespace
-
 # (isCalculated, best fit ci test_env or None)
 BestFitCiTestEnv = (False, None) # type: Tuple[bool, Optional[TestEnv]]
 
@@ -332,7 +358,7 @@ _baseline_depth_commit_log = {} # type: Dict[GitHash, List[GitHash]]
 
 # Get the commit hashes for the last BaselineSearchDepth commits from and
 # including the input commit. The output commits are all commit hashes.
-def baseline_commit_log(commit: GitRef) -> List[GitHash]:
+def baseline_commit_log(commit: Union[GitHash,GitRef]) -> List[GitHash]:
     global _baseline_depth_commit_log
     chash = commit_hash(commit)
     if not commit in _baseline_depth_commit_log:
@@ -374,75 +400,60 @@ _commit_metric_cache = {} # type: ignore
 #                      instead when looking for ci results)
 # metric: str - test metric
 # way: str - test way
-# returns: the Baseline named tuple or None if no metric was found within
+# returns: the Baseline or None if no metric was found within
 #          BaselineSearchDepth commits and since the last expected change
 #          (ignoring any expected change in the given commit).
-def baseline_metric(commit: GitRef,
+def baseline_metric(commit: GitHash,
                     name: TestName,
                     test_env: TestEnv,
                     metric: MetricName,
-                    way: WayName
-                    ) -> Baseline:
+                    way: WayName,
+                    baseline_ref: Optional[GitRef]
+                    ) -> Optional[Baseline]:
     # For performance reasons (in order to avoid calling commit_hash), we assert
     # commit is already a commit hash.
     assert is_commit_hash(commit)
 
     # Get all recent commit hashes.
     commit_hashes = baseline_commit_log(commit)
-    def depth_to_commit(depth):
-        return commit_hashes[depth]
 
-    def has_expected_change(commit):
-        return get_allowed_perf_changes(commit).get(name) \
-                != None
+    baseline_commit = commit_hash(baseline_ref) if baseline_ref else None
 
-    # Bool -> String
-    def namespace(useCiNamespace):
-        return CiNamespace if useCiNamespace else LocalNamespace
-
-    ci_test_env = best_fit_ci_test_env()
-
-    # gets the metric of a given commit
-    # (Bool, Int) -> (float | None)
-    def commit_metric(useCiNamespace, depth):
-        currentCommit = depth_to_commit(depth)
-
-        # Get test environment.
-        effective_test_env = ci_test_env if useCiNamespace else test_env
-        if effective_test_env == None:
-            # This can happen when no best fit ci test is found.
-            return None
-
-        return get_commit_metric(namespace(useCiNamespace), currentCommit, effective_test_env, name, metric, way)
+    def has_expected_change(commit: GitHash) -> bool:
+        return get_allowed_perf_changes(commit).get(name) is not None
 
     # Searches through previous commits trying local then ci for each commit in.
-    def search(useCiNamespace, depth):
-        # Stop if reached the max search depth.
-        # We use len(commit_hashes) instead of BaselineSearchDepth incase
-        # baseline_commit_log() returned fewer than BaselineSearchDepth hashes.
-        if depth >= len(commit_hashes):
-            return None
+    def find_baseline(namespace: NoteNamespace,
+                      test_env: TestEnv
+                      ) -> Optional[Baseline]:
+        if baseline_commit is not None:
+            current_metric = get_commit_metric(namespace, baseline_commit, test_env, name, metric, way)
+            if current_metric is not None:
+                return Baseline(current_metric, baseline_commit)
+            else:
+                return None
 
-        # Check for a metric on this commit.
-        current_metric = commit_metric(useCiNamespace, depth)
-        if current_metric != None:
-            return Baseline(current_metric, depth_to_commit(depth), depth)
+        for depth, current_commit in list(enumerate(commit_hashes))[1:]:
+            # Check for a metric on this commit.
+            current_metric = get_commit_metric(namespace, current_commit, test_env, name, metric, way)
+            if current_metric is not None:
+                return Baseline(current_metric, current_commit)
 
-        # Metric is not available.
-        # If tried local, now try CI.
-        if not useCiNamespace:
-            return search(True, depth)
+            # Stop if there is an expected change at this commit. In that case
+            # metrics on ancestor commits will not be a valid baseline.
+            if has_expected_change(current_commit):
+                return None
 
-        # Stop if there is an expected change at this commit. In that case
-        # metrics on ancestor commits will not be a valid baseline.
-        if has_expected_change(depth_to_commit(depth)):
-            return None
+        return None
 
-        # Move to the parent commit.
-        return search(False, depth + 1)
+    # Test environment to use when comparing against CI namespace
+    ci_test_env = best_fit_ci_test_env()
 
-    # Start search from parent commit using local name space.
-    return search(False, 1)
+    baseline = find_baseline(LocalNamespace, test_env) # type: Optional[Baseline]
+    if baseline is None and ci_test_env is not None:
+        baseline = find_baseline(CiNamespace, ci_test_env)
+
+    return baseline
 
 # Same as get_commit_metric(), but converts the result to a string or keeps it
 # as None.
@@ -467,7 +478,7 @@ def get_commit_metric_value_str_or_none(gitNoteRef,
 # way: test way
 # returns: PerfStat | None if stats don't exist for the given input
 def get_commit_metric(gitNoteRef,
-                      ref: GitRef,
+                      ref: Union[GitRef, GitHash],
                       test_env: TestEnv,
                       name: TestName,
                       metric: MetricName,
@@ -517,7 +528,7 @@ def get_commit_metric(gitNoteRef,
 # tolerance_dev: allowed deviation of the actual value from the expected value.
 # allowed_perf_changes: allowed changes in stats. This is a dictionary as returned by get_allowed_perf_changes().
 # force_print: Print stats even if the test stat was in the tolerance range.
-# Returns a (MetricChange, pass/fail object) tuple. Passes if the stats are withing the expected value ranges.
+# Returns a (MetricChange, pass/fail object) tuple. Passes if the stats are within the expected value ranges.
 def check_stats_change(actual: PerfStat,
                        baseline: Baseline,
                        tolerance_dev,
@@ -555,8 +566,8 @@ def check_stats_change(actual: PerfStat,
     # Print errors and create pass/fail object.
     result = passed()
     if not change_allowed:
-        error = change.value + ' from ' + baseline.perfStat.test_env + \
-                ' baseline @ HEAD~' + str(baseline.commitDepth)
+        error = str(change) + ' from ' + baseline.perfStat.test_env + \
+                ' baseline @ %s' % baseline.commit
         print(actual.metric, error + ':')
         result = failBecause('stat ' + error, tag='stat')
 
@@ -586,20 +597,27 @@ def main() -> None:
     parser.add_argument("--add-note", nargs=3,
                         help="Development only. --add-note N commit seed \
                         Adds N fake metrics to the given commit using the random seed.")
-    parser.add_argument("--chart", nargs='?', default=None, action='store', const='./PerformanceChart.html',
-                        help='Create a chart of the results an save it to the given file. Default to "./PerformanceChart.html".')
     parser.add_argument("--ci", action='store_true',
                         help="Use ci results. You must fetch these with:\n    " \
                             + "$ git fetch https://gitlab.haskell.org/ghc/ghc-performance-notes.git refs/notes/perf:refs/notes/ci/perf")
-    parser.add_argument("--test-env",
-                        help="The given test environment to be compared. Use 'local' for localy run results. If using --ci, see .gitlab-ci file for TEST_ENV settings.")
-    parser.add_argument("--test-name",
-                        help="Filters for tests matching the given regular expression.")
-    parser.add_argument("--metric",
-                        help="Test metric (one of " + str(testing_metrics()) + ").")
-    parser.add_argument("--way",
-                        help="Test way (one of " + str(testing_metrics()) + ").")
-    parser.add_argument("commits", nargs=argparse.REMAINDER,
+
+    group = parser.add_argument_group(title='Filtering', description="Select which subset of performance metrics to dump")
+    group.add_argument("--test-env",
+                       help="The given test environment to be compared. Use 'local' for locally run results. If using --ci, see .gitlab-ci file for TEST_ENV settings.")
+    group.add_argument("--test-name",
+                       help="Filters for tests matching the given regular expression.")
+    group.add_argument("--metric",
+                       help="Test metric (one of " + str(testing_metrics()) + ").")
+    group.add_argument("--way",
+                       help="Test way (one of " + str(testing_metrics()) + ").")
+
+    group = parser.add_argument_group(title='Plotting', description="Plot historical performance metrics")
+    group.add_argument("--chart", nargs='?', default=None, action='store', const='./PerformanceChart.html',
+                       help='Create a chart of the results an save it to the given file. Default to "./PerformanceChart.html".')
+    group.add_argument("--zero-y", action='store_true',
+                       help='When charting, include 0 in y axis')
+
+    parser.add_argument("commits", nargs='+',
                         help="Either a list of commits or a single commit range (e.g. HEAD~10..HEAD).")
     args = parser.parse_args()
 
@@ -614,9 +632,10 @@ def main() -> None:
     # Main logic of the program when called from the command-line.
     #
 
-    ref = 'perf'
+    ref = NoteNamespace('perf')
     if args.ci:
-        ref = 'ci/perf'
+        ref = NoteNamespace('ci/perf')
+
     commits = args.commits
     if args.commits:
         # Commit range
@@ -632,6 +651,8 @@ def main() -> None:
         metrics = [test for test in metrics if test.stat.way == args.way]
 
     if args.test_env:
+        if '"' in args.test_env:
+            raise Exception('test_env should not contain quotation marks')
         metrics = [test for test in metrics if test.stat.test_env == args.test_env]
 
     if args.test_name:
@@ -695,7 +716,13 @@ def main() -> None:
                         'borderColor': hash_rgb_str((env, name, metric, way))
                     } for (env, name, metric, way) in testSeries]
                 },
-                'options': {}
+                'options': {
+                    'scales': {
+                        'yAxes': [{
+                            'ticks': { 'beginAtZero': True }
+                        }]
+                    }
+                }
             }
 
 
