@@ -14,6 +14,7 @@ import tempfile
 import time
 import re
 import traceback
+from pathlib import Path
 
 # We don't actually need subprocess in runtests.py, but:
 # * We do need it in testlibs.py
@@ -24,9 +25,13 @@ import traceback
 import subprocess
 
 from testutil import getStdout, Watcher, str_warn, str_info
-from testglobals import getConfig, ghc_env, getTestRun, TestConfig, TestOptions, brokens
-from perf_notes import MetricChange, inside_git_repo, is_worktree_dirty, format_perf_stat
+from testglobals import getConfig, ghc_env, getTestRun, TestConfig, \
+                        TestOptions, brokens, PerfMetric
+from my_typing import TestName
+from perf_notes import MetricChange, GitRef, inside_git_repo, is_worktree_dirty, format_perf_stat
 from junit import junit
+import term_color
+from term_color import Color, colored
 import cpu_features
 
 # Readline sometimes spews out ANSI escapes for some values of TERM,
@@ -52,6 +57,7 @@ parser = argparse.ArgumentParser(description="GHC's testsuite driver")
 perf_group = parser.add_mutually_exclusive_group()
 
 parser.add_argument("-e", action='append', help="A string to execute from the command line.")
+parser.add_argument("--top", type=Path, help="path to top of testsuite/ tree")
 parser.add_argument("--config-file", action="append", help="config file")
 parser.add_argument("--config", action='append', help="config field")
 parser.add_argument("--rootdir", action='append', help="root of tree containing tests (default: .)")
@@ -64,7 +70,9 @@ parser.add_argument("--skipway", action="append", help="skip this way")
 parser.add_argument("--threads", type=int, help="threads to run simultaneously")
 parser.add_argument("--verbose", type=int, choices=[0,1,2,3,4,5], help="verbose (Values 0 through 5 accepted)")
 parser.add_argument("--junit", type=argparse.FileType('wb'), help="output testsuite summary in JUnit format")
+parser.add_argument("--broken-test", action="append", default=[], help="a test name to mark as broken for this run")
 parser.add_argument("--test-env", default='local', help="Override default chosen test-env.")
+parser.add_argument("--perf-baseline", type=GitRef, metavar='COMMIT', help="Baseline commit for performance comparsons.")
 perf_group.add_argument("--skip-perf-tests", action="store_true", help="skip performance tests")
 perf_group.add_argument("--only-perf-tests", action="store_true", help="Only do performance tests")
 
@@ -93,9 +101,13 @@ if args.rootdir:
     config.rootdirs = args.rootdir
 
 config.metrics_file = args.metrics_file
-hasMetricsFile = bool(config.metrics_file)
+hasMetricsFile = config.metrics_file is not None
 config.summary_file = args.summary_file
 config.no_print_summary = args.no_print_summary
+config.baseline_commit = args.perf_baseline
+
+if args.top:
+    config.top = args.top
 
 if args.only:
     config.only = args.only
@@ -120,6 +132,8 @@ if args.skipway:
     config.run_ways = [w for w in config.run_ways if w not in args.skipway]
     config.compile_ways = [w for w in config.compile_ways if w not in args.skipway]
 
+config.broken_tests |= {TestName(t) for t in args.broken_test}
+
 if args.threads:
     config.threads = args.threads
     config.use_threads = True
@@ -213,6 +227,7 @@ def supports_colors():
     return True
 
 config.supports_colors = supports_colors()
+term_color.enable_color = config.supports_colors
 
 # This has to come after arg parsing as the args can change the compiler
 get_compiler_info()
@@ -264,7 +279,7 @@ testopts_local.x = TestOptions()
 
 # if timeout == -1 then we try to calculate a sensible value
 if config.timeout == -1:
-    config.timeout = int(read_no_crs(config.top + '/timeout/calibrate.out'))
+    config.timeout = int(read_no_crs(config.top / 'timeout' / 'calibrate.out'))
 
 print('Timeout is ' + str(config.timeout))
 print('Known ways: ' + ', '.join(config.other_ways))
@@ -327,6 +342,26 @@ def cleanup_and_exit(exitcode):
         shutil.rmtree(tempdir, ignore_errors=True)
     exit(exitcode)
 
+def tabulate_metrics(metrics: List[PerfMetric]) -> None:
+    for metric in sorted(metrics, key=lambda m: (m.stat.test, m.stat.way, m.stat.metric)):
+        print("{test:24}  {metric:40}  {value:15.3f}".format(
+            test = "{}({})".format(metric.stat.test, metric.stat.way),
+            metric = metric.stat.metric,
+            value = metric.stat.value
+        ))
+        if metric.baseline is not None:
+            val0 = metric.baseline.perfStat.value
+            val1 = metric.stat.value
+            rel = 100 * (val1 - val0) / val0
+            print("{space:24}  {herald:40}  {value:15.3f}  [{direction}, {rel:2.1f}%]".format(
+                space = "",
+                herald = "(baseline @ {commit})".format(
+                    commit = metric.baseline.commit),
+                value = val0,
+                direction = metric.change,
+                rel = rel
+            ))
+
 # First collect all the tests to be run
 t_files_ok = True
 for file in t_files:
@@ -390,20 +425,14 @@ else:
     sys.stdout.flush()
 
     # Dump metrics data.
+    print("\nPerformance Metrics (test environment: {}):\n".format(config.test_env))
+    if config.baseline_commit:
+        print('Performance baseline: %s\n' % config.baseline_commit)
     if any(t.metrics):
-        print("\nPerformance Metrics:\n")
-        for (change, stat, baseline) in t.metrics:
-            if baseline is None:
-                print("{stat} [{direction}]".format(
-                    stat = format_perf_stat(stat),
-                    direction = str(change)))
-            else:
-                print("{stat} [{direction} from ({baselineStat}) @ HEAD~{depth}]".format(
-                    stat = format_perf_stat(stat),
-                    baselineStat = format_perf_stat(baseline.perfStat, ", "),
-                    direction = str(change),
-                    depth = baseline.commitDepth))
-        print("")
+        tabulate_metrics(t.metrics)
+    else:
+        print("\nNone collected.")
+    print("")
 
     # Warn if had to force skip perf tests (see Note force skip perf tests).
     spacing = "       "
@@ -448,25 +477,25 @@ else:
         print(str_info("Some stats have changed") + " If this is expected, " + \
             "allow changes by appending the git commit message with this:")
         print('-' * 25)
-        print(Perf.allow_changes_string(t.metrics))
+        print(Perf.allow_changes_string([(m.change, m.stat) for m in t.metrics]))
         print('-' * 25)
 
     summary(t, sys.stdout, config.no_print_summary, config.supports_colors)
 
     # Write perf stats if any exist or if a metrics file is specified.
-    stats = [stat for (_, stat, __) in t.metrics]
+    stats_metrics = [stat for (_, stat, __) in t.metrics] # type: List[PerfStat]
     if hasMetricsFile:
-        print('Appending ' + str(len(stats)) + ' stats to file: ' + config.metrics_file)
+        print('Appending ' + str(len(stats_metrics)) + ' stats to file: ' + config.metrics_file)
         with open(config.metrics_file, 'a') as f:
-            f.write("\n" + Perf.format_perf_stat(stats))
-    elif inside_git_repo() and any(stats):
+            f.write("\n" + Perf.format_perf_stat(stats_metrics))
+    elif inside_git_repo() and any(stats_metrics):
         if is_worktree_dirty():
             print()
             print(str_warn('Performance Metrics NOT Saved') + \
                 ' working tree is dirty. Commit changes or use ' + \
                 '--metrics-file to save metrics to a file.')
         else:
-            Perf.append_perf_stat(stats)
+            Perf.append_perf_stat(stats_metrics)
 
     # Write summary
     if config.summary_file:
