@@ -4,7 +4,7 @@
 
 from my_typing import *
 from pathlib import Path
-from perf_notes import MetricChange, PerfStat, Baseline, MetricOracles
+from perf_notes import MetricChange, PerfStat, Baseline, MetricOracles, GitRef
 from datetime import datetime
 
 # -----------------------------------------------------------------------------
@@ -22,7 +22,7 @@ class TestConfig:
     def __init__(self):
 
         # Where the testsuite root is
-        self.top = ''
+        self.top = Path('.')
 
         # Directories below which to look for test description files (foo.T)
         self.rootdirs = []
@@ -43,7 +43,7 @@ class TestConfig:
         self.summary_file = ''
 
         # Path to Ghostscript
-        self.gs = ''
+        self.gs = None # type: Optional[Path]
 
         # Run tests requiring Haddock
         self.haddock = False
@@ -115,6 +115,9 @@ class TestConfig:
         self.way_flags = {}  # type: Dict[WayName, List[str]]
         self.way_rts_flags = {}  # type: Dict[WayName, List[str]]
 
+        # Do we have a functional LLVM toolchain?
+        self.have_llvm = False
+
         # Do we have vanilla libraries?
         self.have_vanilla = False
 
@@ -133,12 +136,15 @@ class TestConfig:
         # Do we have SMP support?
         self.have_smp = False
 
-        # Is gdb avaliable?
+        # Is gdb available?
         self.have_gdb = False
 
         # Is readelf available?
         self.have_readelf = False
 
+        # Do we use a fast backend for bignum (e.g. GMP)
+        self.have_fast_bignum = True
+
         # Are we testing an in-tree compiler?
         self.in_tree_compiler = True
 
@@ -156,6 +162,13 @@ class TestConfig:
         self.threads = 1
         self.use_threads = False
 
+        # tests which should be considered to be broken during this testsuite
+        # run.
+        self.broken_tests = set() # type: Set[TestName]
+
+        # Baseline commit for performane metric comparisons.
+        self.baseline_commit = None # type: Optional[GitRef]
+
         # Should we skip performance tests
         self.skip_perf_tests = False
 
@@ -219,6 +232,12 @@ class TestResult:
         self.stdout = stdout
         self.stderr = stderr
 
+# A performance metric measured in this test run.
+PerfMetric = NamedTuple('PerfMetric',
+                        [('change', MetricChange),
+                         ('stat', PerfStat),
+                         ('baseline', Optional[Baseline]) ])
+
 class TestRun:
    def __init__(self) -> None:
        self.start_time = None # type: Optional[datetime]
@@ -246,7 +265,7 @@ class TestRun:
        # [(change, PerfStat)] where change is one of the  MetricChange
        # constants: NewMetric, NoChange, Increase, Decrease.
        # NewMetric happens when the previous git commit has no metric recorded.
-       self.metrics = [] # type: List[Tuple[MetricChange, PerfStat, Optional[Baseline]]]
+       self.metrics = [] # type: List[PerfMetric]
 
 global t
 t = TestRun()
@@ -396,6 +415,7 @@ class TestOptions:
 global default_testopts
 default_testopts = TestOptions()
 
-# (bug, directory, name) of tests marked broken
+# (bug, directory, name) of tests marked broken. Used by config.list_broken
+# feature.
 global brokens
 brokens = []  # type: List[Tuple[IssueNumber, str, str]]
