@@ -19,10 +19,11 @@ import collections
 import subprocess
 
 from testglobals import config, ghc_env, default_testopts, brokens, t, \
-                        TestRun, TestResult, TestOptions
+                        TestRun, TestResult, TestOptions, PerfMetric
 from testutil import strip_quotes, lndir, link_or_copy_file, passed, \
-                     failBecause, str_fail, str_pass, testing_metrics, \
-                     PassFail
+                     failBecause, testing_metrics, \
+                     PassFail, memoize
+from term_color import Color, colored
 import testutil
 from cpu_features import have_cpu_feature
 import perf_notes as Perf
@@ -115,6 +116,12 @@ def expect_fail( name, opts ):
     # future.
     opts.expect = 'fail';
 
+def no_lint( name, opts ):
+   """Disable Core, STG and Cmm lints. Useful when testing compiler perf."""
+   opts.compiler_always_flags = \
+       [opt for opt in opts.compiler_always_flags \
+            if opt not in ['-dcore-lint', '-dstg-lint', '-dcmm-lint']]
+
 def reqlib( lib ):
     return lambda name, opts, l=lib: _reqlib (name, opts, l )
 
@@ -392,7 +399,7 @@ def extra_run_opts( val ):
     return lambda name, opts, v=val: _extra_run_opts(name, opts, v);
 
 def _extra_run_opts( name, opts, v ):
-    opts.extra_run_opts = v
+    opts.extra_run_opts += " " + v
 
 # -----
 
@@ -400,7 +407,7 @@ def extra_hc_opts( val ):
     return lambda name, opts, v=val: _extra_hc_opts(name, opts, v);
 
 def _extra_hc_opts( name, opts, v ):
-    opts.extra_hc_opts = v
+    opts.extra_hc_opts += " " + v
 
 # -----
 
@@ -470,7 +477,8 @@ def _collect_stats(name: TestName, opts, metrics, deviation, is_compiler_stats_t
         metric = '{}/{}'.format(tag, metric_name)
         def baselineByWay(way, target_commit, metric=metric):
             return Perf.baseline_metric( \
-                              target_commit, name, config.test_env, metric, way)
+                              target_commit, name, config.test_env, metric, way, \
+                              config.baseline_commit )
 
         opts.stats_range_fields[metric] = MetricOracles(baseline=baselineByWay,
                                                         deviation=deviation)
@@ -500,7 +508,18 @@ def fast() -> bool:
 def platform( plat: str ) -> bool:
     return config.platform == plat
 
+KNOWN_OPERATING_SYSTEMS = set([
+    'mingw32',
+    'freebsd',
+    'openbsd',
+    'aix',
+    'linux',
+    'darwin',
+    'solaris2',
+])
+
 def opsys( os: str ) -> bool:
+    assert os in KNOWN_OPERATING_SYSTEMS
     return config.os == os
 
 def arch( arch: str ) -> bool:
@@ -545,11 +564,11 @@ def have_gdb( ) -> bool:
 def have_readelf( ) -> bool:
     return config.have_readelf
 
-def integer_gmp( ) -> bool:
-    return have_library("integer-gmp")
+def have_fast_bignum( ) -> bool:
+    return config.have_fast_bignum
 
-def integer_simple( ) -> bool:
-    return have_library("integer-simple")
+def have_slow_bignum( ) -> bool:
+    return not(have_fast_bignum())
 
 def llvm_build ( ) -> bool:
     return config.ghc_built_by_llvm
@@ -569,13 +588,15 @@ def have_thread_sanitizer( ) -> bool:
 # appears to change (up or down) when the underlying profile hasn't
 # really changed. To further minimize this effect we run with a single
 # generation (meaning we get a residency sample on every GC) with a small
-# allocation area (as suggested in #17387).
+# allocation area (as suggested in #17387). That's what +RTS -h -i0 will do.
+# If you find that a test is flaky, sampling frequency can be adjusted by
+# shrinking the allocation area (+RTS -A64k, for example).
 #
 # However, please don't just ignore changes in residency.  If you see
 # a change in one of these figures, please check whether it is real or
 # not as follows:
 #
-#  * Run the test with old and new compilers, adding +RTS -h -i0.01
+#  * Run the test with old and new compilers, adding +RTS -h -i0.001
 #    (you don't need to compile anything for profiling or enable profiling
 #    libraries to get a heap profile).
 #  * view the heap profiles, read off the maximum residency.  If it has
@@ -725,6 +746,30 @@ def normalise_whitespace_fun(f):
 def _normalise_whitespace_fun(name, opts, f):
     opts.whitespace_normaliser = f
 
+def normalise_win32_io_errors(name, opts):
+    """
+    On Windows we currently have two IO manager implementations: both WinIO IO
+    manager and the old POSIX-emulated implementation. These currently differ
+    slightly in the error messages that they provide. Normalise these
+    differences away, preferring the new WinIO errors.
+
+    This can be dropped when the old IO manager is removed.
+    """
+
+    SUBS = [
+        ('Bad file descriptor', 'The handle is invalid'),
+        ('Permission denied', 'Access is denied.'),
+        ('No such file or directory', 'The system cannot find the file specified.'),
+    ]
+
+    def f(s: str):
+        for old,new in SUBS:
+            s = s.replace(old, new)
+
+        return s
+
+    return when(opsys('mingw32'), normalise_fun(f))
+
 def normalise_version_( *pkgs ):
     def normalise_version__( str ):
         return re.sub('(' + '|'.join(map(re.escape,pkgs)) + ')-[0-9.]+',
@@ -861,6 +906,9 @@ def test(name: TestName,
 
     executeSetups([thisdir_settings, setup], name, myTestOpts)
 
+    if name in config.broken_tests:
+        myTestOpts.expect = 'fail'
+
     thisTest = lambda watcher: runTest(watcher, myTestOpts, name, func, args)
     if myTestOpts.alone:
         aloneTests.append(thisTest)
@@ -981,8 +1029,8 @@ def test_common_work(watcher: testutil.Watcher,
             except KeyboardInterrupt:
                 stopNow()
             except Exception as e:
-                framework_fail(name, way, str(e))
                 traceback.print_exc()
+                framework_fail(name, way, traceback.format_exc())
 
         t.n_tests_skipped += len(set(all_ways) - set(do_ways))
 
@@ -1062,7 +1110,7 @@ def do_test(name: TestName,
         dst_makefile = in_testdir('Makefile')
         if src_makefile.exists():
             makefile = src_makefile.read_text(encoding='UTF-8')
-            makefile = re.sub('TOP=.*', 'TOP=' + config.top, makefile, 1)
+            makefile = re.sub('TOP=.*', 'TOP=%s' % config.top, makefile, 1)
             dst_makefile.write_text(makefile, encoding='UTF-8')
 
     if opts.pre_cmd:
@@ -1191,7 +1239,7 @@ def ghci_script( name, way, script):
     # script can invoke the correct compiler by using ':! $HC $HC_OPTS'
     cmd = ('HC={{compiler}} HC_OPTS="{flags}" {{compiler}} {way_flags} {flags}'
           ).format(flags=flags, way_flags=way_flags)
-      # NB: put way_flags before flags so that flags in all.T can overrie others
+      # NB: put way_flags before flags so that flags in all.T can override others
 
     getTestOpts().stdin = script
     return simple_run( name, way, cmd, getTestOpts().extra_run_opts )
@@ -1328,6 +1376,26 @@ def compile_grep_asm(name: TestName,
     # no problems found, this test passed
     return passed()
 
+def compile_grep_core(name: TestName,
+                      way: WayName,
+                      extra_hc_opts: str
+                      ) -> PassFail:
+    print('Compile only, extra args = ', extra_hc_opts)
+    result = simple_build(name + '.hs', way, '-ddump-to-file -dsuppress-all -ddump-simpl -O ' + extra_hc_opts, False, None, False, False)
+
+    if badResult(result):
+        return result
+
+    expected_pat_file = find_expected_file(name, 'substr-simpl')
+    actual_core_file = add_suffix(name, 'dump-simpl')
+
+    if not grep_output(join_normalisers(normalise_errmsg),
+                       expected_pat_file, actual_core_file):
+        return failBecause('simplified core mismatch')
+
+    # no problems found, this test passed
+    return passed()
+
 # -----------------------------------------------------------------------------
 # Compile-and-run tests
 
@@ -1389,7 +1457,7 @@ def metric_dict(name, way, metric, value) -> PerfStat:
 # way: the way.
 # stats_file: the path of the stats_file containing the stats for the test.
 # range_fields: see TestOptions.stats_range_fields
-# Returns a pass/fail object. Passes if the stats are withing the expected value ranges.
+# Returns a pass/fail object. Passes if the stats are within the expected value ranges.
 # This prints the results for the user.
 def check_stats(name: TestName,
                 way: WayName,
@@ -1438,7 +1506,7 @@ def check_stats(name: TestName,
                         tolerance_dev,
                         config.allowed_perf_changes,
                         config.verbose >= 4)
-                t.metrics.append((change, perf_stat, baseline))
+                t.metrics.append(PerfMetric(change=change, stat=perf_stat, baseline=baseline))
 
             # If any metric fails then the test fails.
             # Note, the remaining metrics are still run so that
@@ -1778,10 +1846,10 @@ def read_stdout( name: TestName ) -> str:
         return ''
 
 def dump_stdout( name: TestName ) -> None:
-    str = read_stdout(name).strip()
-    if str:
+    s = read_stdout(name).strip()
+    if s:
         print("Stdout (", name, "):")
-        print(str)
+        safe_print(s)
 
 def stderr_ok(name: TestName, way: WayName) -> bool:
    actual_stderr_file = add_suffix(name, 'run.stderr')
@@ -1800,10 +1868,10 @@ def read_stderr( name: TestName ) -> str:
         return ''
 
 def dump_stderr( name: TestName ) -> None:
-    str = read_stderr(name).strip()
-    if str:
+    s = read_stderr(name).strip()
+    if s:
         print("Stderr (", name, "):")
-        print(str)
+        safe_print(s)
 
 def read_no_crs(f: Path) -> str:
     s = ''
@@ -1859,7 +1927,7 @@ def check_hp_ok(name: TestName) -> bool:
 
     if hp2psResult == 0:
         if actual_ps_path.exists():
-            if gs_working:
+            if does_ghostscript_work():
                 gsResult = runCmd(genGSCmd(actual_ps_path))
                 if (gsResult == 0):
                     return True
@@ -2112,12 +2180,14 @@ def normalise_errmsg(s: str) -> str:
     s = re.sub('runghc-[0-9.]+', 'runghc', s)
     s = re.sub('hpc-[0-9.]+', 'hpc', s)
     s = re.sub('ghc-pkg-[0-9.]+', 'ghc-pkg', s)
+    # hpc executable is given ghc suffix
+    s = re.sub('hpc-ghc', 'hpc', s)
 
-    # Error messages sometimes contain integer implementation package
-    s = re.sub('integer-(gmp|simple)-[0-9.]+', 'integer-<IMPL>-<VERSION>', s)
+    # Error messages sometimes contain ghc-bignum implementation package
+    s = re.sub('ghc-bignum-[0-9.]+', 'ghc-bignum-<VERSION>', s)
 
     # Error messages sometimes contain this blurb which can vary
-    # spuriously depending upon build configuration (e.g. based on integer
+    # spuriously depending upon build configuration (e.g. based on bignum
     # backend)
     s = re.sub('...plus ([a-z]+|[0-9]+) instances involving out-of-scope types',
                  '...plus N instances involving out-of-scope types', s)
@@ -2255,14 +2325,18 @@ def normalise_asm( s: str ) -> str:
           out.append(ins[0])
     return '\n'.join(out)
 
+def safe_print(s: str) -> None:
+    s2 = s.encode(sys.stdout.encoding, errors='replace').decode(sys.stdout.encoding)
+    print(s2)
+
 def if_verbose( n: int, s: str ) -> None:
     if config.verbose >= n:
-        print(s)
+        safe_print(s)
 
 def dump_file(f: Path):
     try:
         with f.open() as file:
-            print(file.read())
+            safe_print(file.read())
     except Exception:
         print('')
 
@@ -2329,31 +2403,38 @@ def runCmd(cmd: str,
 
 # -----------------------------------------------------------------------------
 # checking if ghostscript is available for checking the output of hp2ps
-
 def genGSCmd(psfile: Path) -> str:
     return '{{gs}} -dNODISPLAY -dBATCH -dQUIET -dNOPAUSE "{0}"'.format(psfile)
 
-def gsNotWorking() -> None:
-    global gs_working
-    print("GhostScript not available for hp2ps tests")
+@memoize
+def does_ghostscript_work() -> bool:
+    """
+    Detect whether Ghostscript is functional.
+    """
+    def gsNotWorking(reason: str) -> None:
+        print("GhostScript not available for hp2ps tests:", reason)
 
-global gs_working
-gs_working = False
-if config.have_profiling:
-  if config.gs != '':
-    resultGood = runCmd(genGSCmd(config.top + '/config/good.ps'));
-    if resultGood == 0:
-        resultBad = runCmd(genGSCmd(config.top + '/config/bad.ps') +
-                                   ' >/dev/null 2>&1')
-        if resultBad != 0:
-            print("GhostScript available for hp2ps tests")
-            gs_working = True
-        else:
-            gsNotWorking();
-    else:
-        gsNotWorking();
-  else:
-    gsNotWorking();
+    if config.gs is None:
+        return False
+
+    try:
+        if runCmd(genGSCmd(config.top / 'config' / 'good.ps')) != 0:
+            gsNotWorking("gs can't process good input")
+            return False
+    except Exception as e:
+        gsNotWorking('error invoking gs on bad input: %s' % e)
+        return False
+
+    try:
+        cmd = genGSCmd(config.top / 'config' / 'bad.ps') + ' >/dev/null 2>&1'
+        if runCmd(cmd) == 0:
+            gsNotWorking('gs accepts bad input')
+            return False
+    except Exception as e:
+        gsNotWorking('error invoking gs on bad input: %s' % e)
+        return False
+
+    return True
 
 def add_suffix( name: Union[str, Path], suffix: str ) -> Path:
     if suffix == '':
@@ -2487,18 +2568,16 @@ def summary(t: TestRun, file: TextIO, short=False, color=False) -> None:
         # Only print the list of unexpected tests above.
         return
 
-    colorize = lambda s: s
-    if color:
-        if len(t.unexpected_failures) > 0 or \
-            len(t.unexpected_stat_failures) > 0 or \
-            len(t.unexpected_passes) > 0 or \
-            len(t.framework_failures) > 0:
-            colorize = str_fail
-        else:
-            colorize = str_pass
+    if len(t.unexpected_failures) > 0 or \
+        len(t.unexpected_stat_failures) > 0 or \
+        len(t.unexpected_passes) > 0 or \
+        len(t.framework_failures) > 0:
+        summary_color = Color.RED
+    else:
+        summary_color = Color.GREEN
 
     assert t.start_time is not None
-    file.write(colorize('SUMMARY') + ' for test run started at '
+    file.write(colored(summary_color, 'SUMMARY') + ' for test run started at '
                + t.start_time.strftime("%c %Z") + '\n'
                + str(datetime.datetime.now() - t.start_time).rjust(8)
                + ' spent to go through\n'
