@@ -1,8 +1,9 @@
 import os
-import platform
 import subprocess
 import shutil
+import tempfile
 from pathlib import Path, PurePath
+from term_color import Color, colored
 
 import threading
 
@@ -37,17 +38,11 @@ def strip_quotes(s: str) -> str:
     # Don't wrap commands to subprocess.call/Popen in quotes.
     return s.strip('\'"')
 
-def str_fail(s: str) -> str:
-    return '\033[1m\033[31m' + s + '\033[0m'
-
-def str_pass(s: str) -> str:
-    return '\033[1m\033[32m' + s + '\033[0m'
-
 def str_warn(s: str) -> str:
-    return '\033[1m\033[33m' + s + '\033[0m'
+    return colored(Color.YELLOW, s)
 
 def str_info(s: str) -> str:
-    return '\033[1m\033[34m' + s + '\033[0m'
+    return colored(Color.BLUE, s)
 
 def getStdout(cmd_and_args: List[str]):
     # Can't use subprocess.check_output, since we also verify that
@@ -88,7 +83,31 @@ def testing_metrics():
 #
 # We define the following function to make this magic more
 # explicit/discoverable. You are encouraged to use it instead of os.symlink.
-if os.name == 'nt' and os.getenv('FORCE_SYMLINKS') == None:
+def symlinks_work() -> bool:
+    if os.getenv('FORCE_SYMLINKS') is not None:
+        return True
+    elif os.name == 'nt':
+        # On Windows we try to create a symlink to test whether symlinks are
+        # usable.
+        works = False
+        with tempfile.NamedTemporaryFile() as tmp:
+            try:
+                tmp.write(b'hello')
+                os.symlink(tmp.name, '__symlink-test')
+                works = True
+            except OSError as e:
+                print('Saw {} during symlink test; assuming symlinks do not work.'.format(e))
+            finally:
+                try:
+                    os.unlink('__symlink-test')
+                except:
+                    pass
+
+        return works
+    else:
+        return True
+
+if not symlinks_work():
     def link_or_copy_file(src: Path, dst: Path):
         shutil.copyfile(str(src), str(dst))
 else:
@@ -112,3 +131,16 @@ class Watcher(object):
         if self.pool <= 0:
             self.evt.set()
         self.sync_lock.release()
+
+def memoize(f):
+    """
+    A decorator to memoize a nullary function.
+    """
+    def cached():
+        if cached._cache is None:
+            cached._cache = f()
+
+        return cached._cache
+
+    cached._cache = None
+    return cached
