@@ -6,9 +6,9 @@ module Main where
 
 import System.IO
 import GHC
-import MonadUtils
-import Outputable
-import Bag (filterBag,isEmptyBag)
+import GHC.Utils.Monad
+import GHC.Utils.Outputable
+import GHC.Data.Bag (filterBag,isEmptyBag)
 import System.Directory (removeFile)
 import System.Environment( getArgs )
 
@@ -35,12 +35,12 @@ main = do
         removeFile "Test.hs"
         print ok
     where
-      isDataCon (dL->L _ (AbsBinds { abs_binds = bs }))
+      isDataCon (L _ (AbsBinds { abs_binds = bs }))
         = not (isEmptyBag (filterBag isDataCon bs))
-      isDataCon (dL->L l (f@FunBind {}))
-        | (MG _ (dL->L _ (m:_)) _) <- fun_matches f,
-          ((dL->L _ (c@ConPatOut{})):_)<-hsLMatchPats m,
-          (dL->L l _)<-pat_con c
+      isDataCon (L l (f@FunBind {}))
+        | (MG _ (L _ (m:_)) _) <- fun_matches f,
+          ((L _ (c@ConPat{})):_)<-hsLMatchPats m,
+          (L l _)<-pat_con c
         = isGoodSrcSpan l       -- Check that the source location is a good one
       isDataCon _
         = False
