@@ -1,9 +1,10 @@
 module Main where
 
 import GHC
-import DynFlags
+import GHC.Driver.Session
+import GHC.Driver.Monad
+import GHC.Parser.Lexer (mkParserFlags)
 import System.Environment
-import GhcMonad
 
 testStrings = [
     "import Data.Maybe"
@@ -52,7 +53,8 @@ main = do
   where
     testWithParser parser = do
       dflags <- getSessionDynFlags
-      liftIO . putStrLn . unlines $ map (testExpr (parser dflags)) testStrings
+      let pflags = mkParserFlags dflags
+      liftIO . putStrLn . unlines $ map (testExpr (parser pflags)) testStrings
 
     testExpr parser expr = do
       expr ++ ": " ++ show (parser expr)
