@@ -3,15 +3,15 @@
 -- argument.
 module Main where
 
-import BasicTypes
+import GHC.Types.Basic
 import Data.Data
 import Data.List (intercalate)
 import System.IO
 import GHC
-import DynFlags
-import MonadUtils
-import Outputable
-import Bag (filterBag,isEmptyBag)
+import GHC.Driver.Session
+import GHC.Utils.Monad
+import GHC.Utils.Outputable
+import GHC.Data.Bag (filterBag,isEmptyBag)
 import System.Directory (removeFile)
 import System.Environment( getArgs )
 import qualified Data.Map as Map
