@@ -1,9 +1,9 @@
 {-# LANGUAGE ViewPatterns #-}
 
 import GHC
-import GhcMake
-import DynFlags
-import Finder
+import GHC.Driver.Make
+import GHC.Driver.Session
+import GHC.Driver.Finder
 
 import Control.Monad.IO.Class (liftIO)
 import Data.List (sort, stripPrefix)
