@@ -3,17 +3,17 @@ module Main where
 
 import System.Environment
 
-import NameCache
-import SrcLoc
-import UniqSupply
-import Name
+import GHC.Types.Name.Cache
+import GHC.Types.SrcLoc
+import GHC.Types.Unique.Supply
+import GHC.Types.Name
 
-import HieBin
-import HieTypes
-import HieUtils
+import GHC.Iface.Ext.Binary
+import GHC.Iface.Ext.Types
+import GHC.Iface.Ext.Utils
 
-import DynFlags
-import SysTools
+import GHC.Driver.Session
+import GHC.SysTools
 
 import qualified Data.Map as M
 import Data.Foldable
@@ -42,25 +42,18 @@ dynFlagsForPrinting libdir = do
   systemSettings <- initSysTools libdir
   return $ defaultDynFlags systemSettings (LlvmConfig [] [])
 
-selectPoint :: HieFile -> (Int,Int) -> HieAST Int
-selectPoint hf (sl,sc) = case M.toList (getAsts $ hie_asts hf) of
-    [(fs,ast)] ->
-      case selectSmallestContaining (sp fs) ast of
-        Nothing -> error "point not found"
-        Just ast' -> ast'
-    _ -> error "map should only contain a single AST"
- where
-   sloc fs = mkRealSrcLoc fs sl sc
-   sp fs = mkRealSrcSpan (sloc fs) (sloc fs)
+selectPoint' :: HieFile -> (Int,Int) -> HieAST Int
+selectPoint' hf loc =
+  maybe (error "point not found") id $ selectPoint hf loc
 
 main = do
   libdir:_ <- getArgs
   df <- dynFlagsForPrinting libdir
   nc <- makeNc
-  (hfr, nc') <- readHieFile nc "PatTypes.hie"
+  hfr <- readHieFile (NCU (\f -> pure $ snd $ f nc)) "PatTypes.hie"
   let hf = hie_file_result hfr
   forM_ [p1,p2,p3,p4] $ \point -> do
     putStr $ "At " ++ show point ++ ", got type: "
-    let types = nodeType $ nodeInfo $ selectPoint hf point
+    let types = concatMap nodeType $ getSourcedNodeInfo $ sourcedNodeInfo $ selectPoint' hf point
     forM_ types $ \typ -> do
       putStrLn (renderHieType df $ recoverFullType typ (hie_types hf))
