@@ -1,5 +1,5 @@
 module Package07e where
 import GHC.Hs.MyTypes
-import GHC.Hs.Types
+import GHC.Hs.Type
 import GHC.Hs.Utils
-import UniqFM
+import GHC.Types.Unique.FM
