@@ -1,13 +1,13 @@
 {-# OPTIONS_GHC -Wall #-}
 module Hooks.Plugin (plugin) where
 
-import BasicTypes
-import GhcPlugins
+import GHC.Types.Basic
+import GHC.Plugins
 import GHC.Hs.Expr
 import GHC.Hs.Extension
 import GHC.Hs.Lit
-import Hooks
-import TcRnMonad
+import GHC.Driver.Hooks
+import GHC.Tc.Utils.Monad
 
 plugin :: Plugin
 plugin = defaultPlugin { dynflagsPlugin = hooksP }
