@@ -3,20 +3,21 @@ module Simple.RemovePlugin where
 
 import Control.Monad.IO.Class
 import Data.List (intercalate)
-import Plugins
-import Bag
-import HscTypes
-import TcRnTypes
+import GHC.Driver.Plugins
+import GHC.Data.Bag
+import GHC.Driver.Types
+import GHC.Tc.Types
 import GHC.Hs.Extension
 import GHC.Hs.Expr
-import Outputable
-import SrcLoc
+import GHC.Utils.Outputable
+import GHC.Types.SrcLoc
 import GHC.Hs
 import GHC.Hs.Binds
-import OccName
-import RdrName
-import Name
-import Avail
+import GHC.Types.Name.Occurrence
+import GHC.Types.Name.Reader
+import GHC.Types.Name
+import GHC.Types.Avail
+import GHC.Hs.Dump
 
 plugin :: Plugin
 plugin = defaultPlugin { parsedResultAction = parsedPlugin
@@ -31,8 +32,8 @@ parsedPlugin [name, "parse"] _ pm
   = return $ pm { hpm_module = removeParsedBinding name (hpm_module pm) }
 parsedPlugin _ _ pm = return pm
 
-removeParsedBinding :: String -> Located (HsModule GhcPs)
-                         -> Located (HsModule GhcPs)
+removeParsedBinding :: String -> Located HsModule
+                         -> Located HsModule
 removeParsedBinding name (L l m)
   = (L l (m { hsmodDecls = filter (notNamedAs name) (hsmodDecls m) } ))
   where notNamedAs name (L _ (ValD _ (FunBind { fun_id = L _ fid })))
@@ -52,10 +53,13 @@ typecheckPlugin [name, "typecheck"] _ tc
 typecheckPlugin _ _ tc = return tc
 
 metaPlugin' :: [CommandLineOption] -> LHsExpr GhcTc -> TcM (LHsExpr GhcTc)
-metaPlugin' [name, "meta"] (L _ (HsApp noExt (L l (HsVar _ (L _ id))) e))
+metaPlugin' [name, "meta"] (L l (HsPar x (L _ (XExpr (WrapExpr (HsWrap w (HsApp noExt (L _ (HsVar _ (L _ id))) e)))))))
   | occNameString (getOccName id) == name
-  = return e
-metaPlugin' _ meta = return meta
+  = return (L l (XExpr (WrapExpr (HsWrap w (unLoc e)))))
+-- The test should always match this first case. If the desugaring changes
+-- again in the future then the panic is more useful than the previous
+-- inscrutable failure.
+metaPlugin' _ meta = pprPanic "meta" (showAstData BlankSrcSpan meta)
 
 interfaceLoadPlugin' :: [CommandLineOption] -> ModIface -> IfM lcl ModIface
 interfaceLoadPlugin' [name, "interface"] iface
