@@ -14,20 +14,24 @@ import Data.Kind
 
 {-# RULES
 "example"  forall a b. forall. map @a @b f = f
-"example2" forall a. forall (x :: a). id x = x
+"example2" forall a. forall (x :: a). id' x = x
   #-}
 
 {-# NOINLINE f #-}
 f :: a -> b
 f = undefined
 
+id' :: a -> a
+id' x = x
+{-# NOINLINE id' #-}
+
 -- More tests
 
 {-# RULES
 "example3" forall (a :: Type -> Type) (b :: a Int) c. forall x y. g @(Proxy b) @(Proxy c) x y = ()
-"example4" forall (a :: Bool) (b :: Proxy a). forall x. g @(Proxy b) @() x = id @()
-"example5" forall (a :: Type). forall. h @a = id @a
-"example5" forall k (c :: k). forall (x :: Proxy c). id @(Proxy c) x = x
+"example4" forall (a :: Bool) (b :: Proxy a). forall x. g @(Proxy b) @() x = id' @()
+"example5" forall (a :: Type). forall. h @a = id' @a
+"example5" forall k (c :: k). forall (x :: Proxy c). id' @(Proxy c) x = x
   #-}
 
 {-# NOINLINE g #-}
@@ -42,4 +46,4 @@ h x = x
 {-# RULES "example6" forall a forall. g a forall = () #-}
 
 -- Should generate a warning
-{-# RULES "example7" forall a b. forall (x :: a). id x = x #-}
+{-# RULES "example7" forall a b. forall (x :: a). id' x = x #-}
